#ifndef _READER_HYPERTEXT_H
#define _READER_HYPERTEXT_H
/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	definition of a HyperText object

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 17 November 1995
*/

#include "mem_h"
#include "io_h"
#include "PROforma_h"
#include "reader_h"
#include "style_h"

typedef struct _LinkArea LinkArea;
typedef struct _Marker Marker;
typedef struct _StylePage StylePage;
typedef struct _StyleLine StyleLine;
typedef struct _Link Link;
typedef struct _Contents Contents;
typedef struct _LineElement LineElement;

#define MAXLINKPOSITION 	32
#define TITLEBUFFER 		80
#define BUFFERSIZE			200
#define HYPERTEXT_BLOCKSIZE 2048

struct _TextLine {
	TextLine *next, *prev;			/* links to next and previous lines */
	LinkArea *links;				/* the links in this line */
	Marker *markers;				/* the markers for this line */
	pt height;						/* height of this line */
	pt baseline;					/* baseline position in the line */
	StyleLine *styleline;			/* font and fontsize at start of line */
									/* only used to start display anywhere */
	LineElement *elements;			/* the elements in the line */
	LineElement *last;				/* last element or NULL - for fast linki */
};

struct _HyperText {
	Gstate gstate;					/* USE ONLY WHILE BUILDING */
	Global *global; 				/* global data structure */
	Style *look;					/* the general look for this text */
	StylePage *style;				/* current stylepage (tree) */
	TextLine lines; 				/* the lines which build the column */
									/* double linked - this is head/sentinel */
	StyleLine *fonts;				/* the font definitions which are used */
	Link *links;					/* the links which are in the file */
	Heap heap;						/* heap for allocating all the memory */
	char title[TITLEBUFFER];		/* title for document */
	int posintitle; 				/* next character to write title */
	char buffer[BUFFERSIZE+2];		/* buffer to keep some data before store */
	int bufferpos;					/* pos for next character in buffer */
	pt posinline;					/* position in line */
	pt width, height;				/* size of text - for scroll bars */
	Link *linkcurr; 				/* currentlink */
	pt linkstart;					/* xorg of start of link */
	pt colour;						/* main drawing colour */

	Chunks cont_chunks; 			/* chunks with contents */
	Contents *cont_first;			/* first file in list */
	Contents *cont_curr;			/* current file in contents */
	Contents *cont_insert;			/* place to insert new files */
	char directory[IO_MAXDIRECTORY];
};

struct _LinkArea {
	/* clickable area falls inside the line */
	/* the height matches the height of the line */

	LinkArea *next; 				/* next clickable area in this line */
									/* areas are linked from right to left ! */
	pt xorg, xend;					/* start and end coordinate of area */
	Link *destination;				/* the destination of the link */
};

struct _Link {
	char filename[IO_MAXFULLNAME];	/* file to link to */
	char position[MAXLINKPOSITION]; /* position in file to link to */
};

struct _Marker {
	Marker *next;					/* next marker for this line */
	char name[MAXLINKPOSITION]; 	/* name for this marker */
};

struct _StylePage {
	StylePage *next;				/* previous stylepage cfr. stack */
	pt marginleft, marginright; 	/* left and right margins for each line */
	StyleLine *font;				/* current font and size */
	pt height;						/* line height for current font */
	pt baseline;					/* baseline position for current font */
	char wrap;						/* indicate whether lines should wrap */
	/* ..... */
	int parsemode;					/* used by "HTML_c" parsemode */
	int context;					/* used by "HTML_c" context */
	int level;						/* used by "HTML_c" level for HTML_OL */
};

struct _StyleLine {
	StyleLine *next;				/* next styleline which is in use */
	char *font; 					/* the font which is used for this line */
	pt size;						/* point size of font at start of line */
};

struct _LineElement {
	LineElement *next;				/* next element in line */
	Error (*Draw)(Gstate, LineElement *);	/* how to draw this element */
	/* ..... */ 					/* rest filled in for use by Draw method */
};

struct _Contents {
	Contents *next;
	char filename[IO_MAXFULLNAME];
	char directory[IO_MAXDIRECTORY];
};

/* the functions to manipulate a HyperText object */

/* (re)initialise a HyperText object - make it available to build */
/* also initialises the bodyfont */
Error HTReInit(Global *global, Gstate gstate, Style *look, HyperText **text, pt columnwidth);

/* add a character in the HyperText object. The character will be displayed */
/* using the currently valid style. The text will fall inside the margins. */
Error HTAddCharacter(HyperText *text, char ch);

/* change the font and/or fontsize which are used. No other effect */
/* font==NULL -> keep current font, size==0 -> keep current size */
Error HTChangeFont(HyperText *text, FontStyle font, short size);

/* change the margins. Normally, you should do a HTLineBreak beforehand. */
/* The margins for the current line are adjusted. The next character will be */
/* displayed at the left margin in the line */
Error HTChangeMargin(HyperText *text, short left, short right);

/* Force a line break. The next characters will display no the next line */
Error HTLineBreak(HyperText *text);

/* Force a line break and insert some whitespace (vertically) */
Error HTSpaceVertical(HyperText *text, short dist);

/* insert some (horizontal) whitespace */
Error HTSpaceHorizontal(HyperText *text, short dist);

/* force the creation of a new stylepage (stylepage are kept in a stack) */
Error HTStyleNew(HyperText *text);

/* restore previous stylepage (stylepage are kept in a stack) */
Error HTStyleRestore(HyperText *text);

/* end the hypertext document - flushes all buffers */
Error HTEnd(HyperText *text);

/* indicate whether line wrap should be used - TRUE or FALSE, default TRUE */
Error HTLineWrap(HyperText *text, char status);

/* insert a horizontal line - conforming to the stylepage */
Error HTLineHorizontal(HyperText *text);

/* start or end a link element */
Error HTLinkStart(HyperText *text, char *filename, char *position);
Error HTLinkEnd(HyperText *text);

/* mark a position in the file - to link to */
Error HTLinkMarker(HyperText *text, char *position);

/* add a file to and delete the contents list */
Error HTContentsAdd(HyperText *text, char *filename);
Error HTContentsDelete(HyperText *text);

/* Put an image in the hypertext file. Size is given in PROforma */
/* coordinates. If both are zero, then the routine will use some */
/* default rule to determine the size. */
Error HTImage(HyperText *text, char *file, pt xsize, pt ysize);

#endif _READER_HYPERTEXT_H
