/*
	ProWesS reader - a HTML file browser

	PROforma, PROGS Font & Raster Manager
	ProWesS, PROGS Windowing System

	definition of the bookmark handling object

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 12 December 1995
*/

#include "str_h"

#include "bookmark_h"
#include "HyperText_h"

/* external functions */
Error DisplayError(Error error);

/* configurable items */
extern char DefaultBookmarks[];
extern char ItemBookmarkSave[];

typedef struct _Element {
	struct _Element *next;
	char filename[IO_MAXFULLNAME];
	char directory[IO_MAXDIRECTORY];
	char title[TITLEBUFFER];
} Element;

typedef struct _BookMarks {
	Chunks chunks;
	Element *first;
	Element *last;
};

#define MAXLINE IO_MAXFULLNAME+IO_MAXDIRECTORY+TITLEBUFFER+20

Error BookmarkSelect(PWObject object, char *item)
{
	Global *g;
	BookMarks *b;
	Error err;
	Element *el=(Element *)(item-offsetof(Element,title));

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	if (!(b=g->bookmarks)) return ERR_OK;

	STRCopy(el->filename,g->filename);
	STRCopy(el->directory,g->directory);
	g->loaded=TRUE;
	PWChange(object,PW_SYSTEM_BREAKDOWN,NULL);

	return ERR_OK;
}

Error BookmarkMenu(Global *g, PWObject object)
{
	BookMarks *b;
	Error err;
	Element *el;

	if (!(b=g->bookmarks)) return ERR_OK;

	PWChange(object,PW_MENU_CLEAR,NULL);
	for (el=b->first ; el ; el=el->next)
		PWChange(object,PW_MENU_ADD,el->title,NULL);

	return ERR_OK;
}

Error BookmarkAdd(PWObject object)
{
	Global *g;
	BookMarks *b;
	Error err;
	Element *el;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	if (!g->text || !(b=g->bookmarks)) return ERR_OK;

	err=MEMUserAllocate(b->chunks,&el);
	if (err) return ERR_OK;

	el->next=NULL;
	STRCopy(g->filename,el->filename);
	STRCopy(g->directory,el->directory);
	STRCopy(g->text->title,el->title);

	if (b->last)
		b->last->next=el;
	else
		b->first=el;
	b->last=el;

	BookmarkMenu(g,g->menu);

	return ERR_OK;
}

static ERROR_BUILD(NoBookmarkFile,"not a bookmark file");
static ERROR_BUILD(EarlyEOF,"incomplete line");
#define HEADER "--- ProWesS reader bookmark list ---"

static Error load(BookMarks *b, char *filename, char *directory, char init)
{
	Channel file;
	Error err;
	char line[MAXLINE];
	char **strs;
	Element *el, *el2;

	err=IOOpenPath(filename,OPEN_OLD,directory,&file);
	if (err) { if (init) return ERR_OK; else return DisplayError(err); }

	err=IOGetString(file, line, MAXLINE);
	if (err) return DisplayError(err);

	if (!STRSameCD(line,HEADER))
		if (err) return DisplayError(ERROR_CODE(NoBookmarkFile));

	/* release previous bookmarks */
	for (el=b->first ; el ; )
	{
		el2=el;
		el=el->next;
		MEMUserRelease(b->chunks,el2);
	}
	b->first=b->last=NULL;

	while(!IOTest(file))
	{
		IOGetString(file, line, MAXLINE);
		err=STRBreak(line,&strs);
		if (err) { IOClose(file); return(DisplayError(err)); }
		if (!strs[0] || !strs[1] || !strs[2])
		{
			MEMRelease(strs);
			IOClose(file);
			return DisplayError(ERROR_CODE(EarlyEOF));
		}

		/* add bookmark */
		err=MEMUserAllocate(b->chunks,&el);
		if (err) { IOClose(file); return DisplayError(err); }
		el->next=NULL;
		STRCopySafe(strs[0],el->title,TITLEBUFFER);
		STRCopySafe(strs[1],el->filename,IO_MAXFULLNAME);
		STRCopySafe(strs[2],el->directory,IO_MAXDIRECTORY);
		MEMRelease(strs);
		if (b->last)
			b->last->next=el;
		else
			b->first=el;
		b->last=el;
	}

	IOClose(file);
	return err;
}

Error bmload(PWObject object, char *directory, char *filename)
{
	BookMarks *b;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&b);
	return load(b,filename,directory,FALSE);
}

Error BookmarkLoad(PWObject object)
{
	Global *g;
	BookMarks *b;
	Error err;
	PWObject fsel;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	if (!(b=g->bookmarks)) return ERR_OK;

	err=PWCreate(NULL,&fsel,PW_TYPE_FILESELECT,
					PW_FILESELECT_ACTION, bmload,
					PW_FILESELECT_EXTENSION, "_dat",
					PW_GLOBAL_AUXILIARY, b,
					NULL);
	if (err) return ERR_OK;
	err=PWChange(fsel,PW_FILESELECT_ACTIVATE,NULL);
	if (err) { PWRemove(fsel); return ERR_OK; }
	BookmarkMenu(g,g->menu);

	PWRemove(fsel);
	return ERR_OK;
}

Error save2(BookMarks *b, Channel file)
{
	Error err;
	char line[MAXLINE];
	Element *el;

	catch( IOPutString(file,HEADER,NULL) );

	for (el=b->first ; el ; el=el->next)
	{
		char *from=el->title;
		char *to=line;

		*to++='\"';
		while (*from)
			if (*from!='\"') *to++=*from++;
			else *to++='\"', *to++='\"';
		*to++='\"'; *to++=' '; *to++='\"';
		from=el->filename;
		while (*from)
			if (*from!='\"') *to++=*from++;
			else *to++='\"', *to++='\"';
		*to++='\"'; *to++=' '; *to++='\"';
		from=el->directory;
		while (*from)
			if (*from!='\"') *to++=*from++;
			else *to++='\"', *to++='\"';
		*to++='\"'; *to++='\0';

		catch( IOPutString(file,line,NULL) );
	}

	return err;
}

Error save(PWObject object)
{
	Channel file;
	Error err;
	char filename[IO_MAXFULLNAME];
	BookMarks *b;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&b);
	PWChange(object,PW_EDLINE_GET,IO_MAXFULLNAME,filename,NULL);

	err=IOOpenPath(filename,OPEN_OVER,PATH_DATA,&file);
	if (err) DisplayError(err);

	err=save2(b,file);
	IOClose(file);
	if (err) DisplayError(err);
	return PWChange(object,PW_SYSTEM_BREAKDOWN,NULL);
}

Error BookmarkSave(PWObject object)
{
	Global *g;
	BookMarks *b;
	Error err;
	PWObject obj;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);

	if (!(b=g->bookmarks) || !b->first) return ERR_OK;

	err=PWCreate(NULL,&obj,PW_TYPE_OUTLINE,
					PW_OUTLINE_QUIT,
					PW_OUTLINE_QUIT_KEYPRESS, 27,
					PW_OUTLINE_TITLE_TEXT, ItemBookmarkSave,
					NULL);
	err=PWCreate(obj,NULL,PW_TYPE_DIRECT_EDLINE,
					PW_EDLINE_SET, DefaultBookmarks,
					PW_EDLINE_MAXLENGTH, IO_MAXFULLNAME,
					PW_EDLINE_ACTION_AFTER, save,
					PW_GLOBAL_AUXILIARY, b,
					NULL);
	if (err) return DisplayError(err);
	err=PWActivate(obj);
	if (err) err=DisplayError(err);
	PWRemove(obj);

	return ERR_OK;
}

Error BookmarkInit(Global *g)
{
	Error err;
	BookMarks *b;

	catch( MEMAllocate(sizeof(BookMarks),&b) );
	catch( MEMUserInit(sizeof(Element),10,&b->chunks) );
	g->bookmarks=b;
	load(b,DefaultBookmarks,"", TRUE);
	return err;
}
